#ifndef POKEPLATINUM_OV95_02246C20_H
#define POKEPLATINUM_OV95_02246C20_H

#include "struct_decls/struct_020067E8_decl.h"
#include "struct_decls/struct_0200B144_decl.h"
#include "struct_decls/struct_0200B358_decl.h"
#include "struct_decls/struct_02018340_decl.h"
#include "struct_decls/struct_02022550_decl.h"
#include "struct_defs/box_pokemon.h"
#include "overlay019/struct_ov19_021DA864.h"
#include "overlay095/struct_ov95_02247004_decl.h"
#include "overlay095/struct_ov95_022472C4_decl.h"
#include "overlay095/struct_ov95_02247568.h"
#include "overlay095/struct_ov95_02247628_decl.h"

#include <nnsys.h>

int ov95_02246C20(UnkStruct_020067E8 * param0, int * param1);
int ov95_02246E1C(UnkStruct_020067E8 * param0, int * param1);
int ov95_02246E7C(UnkStruct_020067E8 * param0, int * param1);
UnkStruct_ov95_02247004 * ov95_02246F30(BOOL * param0, int param1);
void  ov95_02247004(UnkStruct_ov95_02247004 * param0);
void  ov95_02247018(UnkStruct_ov95_02247004 * param0);
UnkStruct_ov95_022472C4 * ov95_022472C4(UnkStruct_02018340 * param0, fx32 param1, fx32 param2, fx32 param3, fx32 param4, int param5, volatile BOOL * param6);
void ov95_022473E8(UnkStruct_ov95_02247628 * param0, int param1, u32 param2, u32 param3, BOOL param4);
void ov95_022474D4(UnkStruct_ov95_02247628 * param0, int param1, u32 param2, u32 param3, u32 param4, u32 param5);
void ov95_02247568(UnkStruct_ov95_02247568 * param0, u32 param1, u32 param2, u32 param3);
void ov95_022475A0(UnkStruct_ov95_02247568 * param0);
void ov95_022475C4(UnkStruct_ov19_021DA864 * param0, UnkStruct_ov95_02247568 * param1, NNSG2dImageProxy * param2, NNSG2dImagePaletteProxy * param3, u32 param4);
UnkStruct_02022550 * ov95_022475E4(UnkStruct_ov95_02247628 * param0, UnkStruct_ov19_021DA864 * param1, u32 param2, u32 param3, u32 param4, int param5);
UnkStruct_02018340 * ov95_02247628(UnkStruct_ov95_02247628 * param0);
UnkStruct_0200B358 * ov95_0224762C(UnkStruct_ov95_02247628 * param0);
UnkStruct_0200B144 * ov95_02247630(UnkStruct_ov95_02247628 * param0);
const BoxPokemon * ov95_02247634(UnkStruct_ov95_02247628 * param0);
const BoxPokemon * ov95_0224763C(UnkStruct_ov95_02247628 * param0);
u32 ov95_02247644(UnkStruct_ov95_02247628 * param0);
u16 ov95_0224764C(UnkStruct_ov95_02247628 * param0);
u16 ov95_02247654(UnkStruct_ov95_02247628 * param0);
u16 ov95_02247660(UnkStruct_ov95_02247628 * param0);
u16 ov95_02247668(UnkStruct_ov95_02247628 * param0);
int ov95_02247674(UnkStruct_ov95_02247628 * param0);
int ov95_02247680(UnkStruct_ov95_02247628 * param0);
void ov95_022476C8(void * param0);

#endif // POKEPLATINUM_OV95_02246C20_H
